@@ -15,6 +15,7 @@ new_vm_inst_command( VM_CMD cmd )
 	temp_inst->prev = NULL;
 	temp_inst->next = NULL;
 	temp_inst->last = temp_inst;
+	temp_inst->loc = loc_init();
 	return temp_inst;
 }
 
@@ -208,7 +209,6 @@ vm_inst_list_cat( vm_inst_list* list1, vm_inst_list* list2 )
 void
 vm_inst_list_show_all( vm_inst_list* list )
 {
-	char* curr_cmd;
 	vm_inst* curr_inst;
 	curr_inst = list;
     DEBUG_PRINT("For ptr_table record, VM instructions just holds 'key name'.\n");
@@ -273,6 +273,15 @@ vm_inst_list_to_code( vm_inst_list* list)
 }
 
 
+// Used to set script location to vm instruction
+void
+vm_inst_set_loc_to_last( struct script_loc loc, vm_inst* insts)
+{
+	insts->last->loc = loc;
+}
+
+
+
 // main function to check behavior 
 // gcc gen_code_util.c vm/vm_code.o vm/vm_cmd.o -Ivm
 
