@@ -10,6 +10,7 @@
 #include "vm_item_pp2val.h"
 #include "ptr_table.h"
 #include "helper.h"
+#include "vm_error.h"
 
 #define INT_OVERFLOW
 
@@ -177,8 +178,6 @@ item_is_nan (stack_item* item)
 int
 vm_calc_addx(vm_stack* vmstack, ptr_table** table)
 {
-	int sp = vmstack->sp;
-	stack_item* stack = vmstack->stack;
 	stack_item* top_item = vm_stack_top(vmstack);
 	stack_item* sec_item = vm_stack_second(vmstack);
 	stack_item_pp2value( top_item );
@@ -198,16 +197,10 @@ vm_calc_addx(vm_stack* vmstack, ptr_table** table)
 		temp_dval = dbl_add ( (double) sec_item->ival , (double) top_item->ival ) ; 
 		if( within_int_limits( temp_dval )){ 
 			result_ival = int_add ( sec_item->ival , top_item->ival ) ; 
-			// vmstack->sp = vmstack->sp - 1;  
-			// sec_item->type = IVAL; 
-			// sec_item->ival = result_ival; 
 			vm_stack_clean_and_pop( vmstack, 2);
 			vm_stack_push_ival( vmstack, result_ival );
 		}else{ 
-			result_dval = temp_dval ; 
-			// vmstack->sp = vmstack->sp - 1;  
-			// sec_item->type = DVAL; 
-			// sec_item->dval = result_dval; 
+			result_dval = temp_dval ;  
 			vm_stack_clean_and_pop( vmstack, 2);
 			vm_stack_push_dval( vmstack, result_dval );
 		} 
@@ -224,12 +217,15 @@ vm_calc_addx(vm_stack* vmstack, ptr_table** table)
 			str1 = (* (sec_item->pp_str)); /* Left on ADDX operator */
 			str2 = (* (top_item->pp_str)); /* Right on ADDX operator */
 
-			new_pp_str = (string_object**) string_ptr_concat( str1, str2 );
+			new_p_str = (string_object*) string_ptr_concat( str1, str2 );
+			new_pp_str = (string_object**) malloc(sizeof(string_object*));
+			*new_pp_str = new_p_str;
 			vm_stack_clean_and_pop( vmstack, 2);
 			vm_stack_push_temp_pp_str( vmstack, new_pp_str);
 			return 1;
 		}else{
 			Rprintf("ERROR: ADDX should be applied to 'num and num' or 'str and str' on stack.\n");
+			vm_error_raise(vmstack);
 			return 0;
 		}
 		vm_stack_clean_and_pop( vmstack, 2);
@@ -240,8 +236,6 @@ vm_calc_addx(vm_stack* vmstack, ptr_table** table)
 
 #define DEFINE_BINARY_OPERATOR( INTFUNC , DBLFUNC ) \
 do { \
-	int sp = vmstack->sp; \
-	stack_item* stack = vmstack->stack; \
 	stack_item* top_item = vm_stack_top(vmstack); \
 	stack_item* sec_item = vm_stack_second(vmstack); \
 	stack_item_pp2value( top_item ); \
@@ -272,7 +266,8 @@ do { \
 		}else if((top_item->type == DVAL) && (sec_item->type == DVAL)){ \
 			result_dval = DBLFUNC ( sec_item->dval , top_item->dval ) ; \
 		}else{ \
-			Rprintf("This VM_CMD should be applied to num and num on stack.\n"); \
+			Rprintf("ERROR: This VM_CMD should be applied to num and num on stack.\n"); \
+			vm_error_raise(vmstack); \
 			return 0; \
 		} \
 		vm_stack_clean_and_pop( vmstack, 2); \
@@ -309,14 +304,12 @@ vm_calc_modx(vm_stack* vmstack)
 int
 vm_calc_divx(vm_stack* vmstack)
 {
-	int sp = vmstack->sp;
-	stack_item* stack = vmstack->stack;
 	stack_item* top_item = vm_stack_top(vmstack);
 	stack_item* sec_item = vm_stack_second(vmstack);
 	stack_item_pp2value( top_item );
 	stack_item_pp2value( sec_item );
 
-	int result_ival;
+//	int result_ival;  // divx always return double
 	double result_dval;
 	// For ivals
 	if((top_item->type == IVAL) && (sec_item->type == IVAL)){
@@ -345,6 +338,7 @@ vm_calc_divx(vm_stack* vmstack)
 			result_dval = dbl_div( sec_item->dval , top_item->dval ) ;
 		}else{
 			Rprintf("ERROR: DIVX should be applied to num and num on stack.\n");
+			vm_error_raise(vmstack);
 			return 0;
 		}
 		vmstack->sp = vmstack->sp - 1; 
@@ -358,8 +352,6 @@ vm_calc_divx(vm_stack* vmstack)
 int
 vm_calc_factorial(vm_stack* vmstack)
 {
-	int sp = vmstack->sp;
-	stack_item* stack = vmstack->stack;
 	stack_item* top_item = vm_stack_top(vmstack);
 	stack_item_pp2value( top_item );
 
@@ -390,6 +382,7 @@ vm_calc_factorial(vm_stack* vmstack)
 			top_item->ival = result_ival;		
 	} else {
 		Rprintf("ERROR: FACT should be applied to num and num on stack.\n");
+		vm_error_raise(vmstack);
 		return 0;
 	}
 	return 1;
@@ -398,8 +391,6 @@ vm_calc_factorial(vm_stack* vmstack)
 // unitary minus (single operator)
 int vm_calc_uminus(vm_stack* vmstack)
 {
-	int sp = vmstack->sp;
-	stack_item* stack = vmstack->stack;
 	stack_item* top_item = vm_stack_top(vmstack);
 	stack_item_pp2value( top_item );
 
@@ -420,6 +411,7 @@ int vm_calc_uminus(vm_stack* vmstack)
 			top_item->dval = result_dval;		
 	} else {
 		Rprintf("ERROR: uminus should be applied to num and num on stack.\n");
+		vm_error_raise(vmstack);
 		return 0;
 	}
 	return 1;
@@ -430,8 +422,6 @@ int vm_calc_uminus(vm_stack* vmstack)
 int
 vm_calc_and(vm_stack* vmstack)
 {
-	int sp = vmstack->sp;
-	stack_item* stack = vmstack->stack;
 	stack_item* top_item = vm_stack_top(vmstack);
 	stack_item* sec_item = vm_stack_second(vmstack);
 	bool result_bool;
@@ -443,6 +433,7 @@ vm_calc_and(vm_stack* vmstack)
 		sec_item->boolean = result_bool;
 	}else{
 		Rprintf("ERROR: AND should be applied to boolean and boolean.\n");
+		vm_error_raise(vmstack);
 		return 0;
 	}
 	return 1;
@@ -451,8 +442,6 @@ vm_calc_and(vm_stack* vmstack)
 int
 vm_calc_or(vm_stack* vmstack)
 {
-	int sp = vmstack->sp;
-	stack_item* stack = vmstack->stack;
 	stack_item* top_item = vm_stack_top(vmstack);
 	stack_item* sec_item = vm_stack_second(vmstack);
 	bool result_bool;
@@ -464,6 +453,7 @@ vm_calc_or(vm_stack* vmstack)
 		sec_item->boolean = result_bool;
 	}else{
 		Rprintf("ERROR: AND should be applied to boolean and boolean.\n");
+		vm_error_raise(vmstack);
 		return 0;
 	}
 	return 1;
@@ -472,8 +462,6 @@ vm_calc_or(vm_stack* vmstack)
 int
 vm_calc_eq(vm_stack* vmstack)
 {
-	int sp = vmstack->sp;
-	stack_item* stack = vmstack->stack;
 	stack_item* top_item = vm_stack_top(vmstack);
 	stack_item* sec_item = vm_stack_second(vmstack);
 	stack_item_pp2value( top_item ); 
@@ -501,11 +489,18 @@ vm_calc_eq(vm_stack* vmstack)
 			result_bool = ( sec_item->ival == top_item->dval ) ;
 		}else if((top_item->type == DVAL) && (sec_item->type == DVAL)){
 			result_bool = ( sec_item->dval == top_item->dval ) ;
+		}else{
+			result_bool = false;
+			Rprintf("ERROR: This branch should not be executed.\n");
+			vm_error_raise(vmstack);
+			return 0;
 		}
 	}else if(item_is_str(sec_item) && item_is_str(top_item)){
 		result_bool = string_compare( *(sec_item->pp_str), *(top_item->pp_str) ) ;
 	}else{
+		result_bool = false;
 		Rprintf("ERROR: Types are invalied for VM_EQ command.\n");
+		vm_error_raise(vmstack);
 		return 0;
 	}
 
@@ -521,8 +516,6 @@ vm_calc_eq(vm_stack* vmstack)
 int
 vm_calc_neq(vm_stack* vmstack)
 {
-	int sp = vmstack->sp;
-	stack_item* stack = vmstack->stack;
 	stack_item* top_item = vm_stack_top(vmstack);
 	stack_item* sec_item = vm_stack_second(vmstack);
 	stack_item_pp2value( top_item ); 
@@ -550,11 +543,18 @@ vm_calc_neq(vm_stack* vmstack)
 			result_bool = ( sec_item->ival != top_item->dval ) ;
 		}else if((top_item->type == DVAL) && (sec_item->type == DVAL)){
 			result_bool = ( sec_item->dval != top_item->dval ) ;
+		}else{
+			result_bool = false;
+			Rprintf("ERROR: This branch should not be executed.\n");
+			vm_error_raise(vmstack);
+			return 0;
 		}
 	}else if(item_is_str(sec_item) && item_is_str(top_item)){
 		result_bool = !(string_compare( *(sec_item->pp_str), *(top_item->pp_str) )) ;
 	}else{
+		result_bool = true;  // Currently comparing incompatible types returns true, though this should be an error case. 
 		Rprintf("ERROR: Types are invalied for VM_EQ command.\n");
+		vm_error_raise(vmstack);
 		return 0;
 	}
 
@@ -568,8 +568,6 @@ vm_calc_neq(vm_stack* vmstack)
 
 #define DEFINE_LOGICAL_OPERATOR( OP ) \
 do { \
-	int sp = vmstack->sp; \
-	stack_item* stack = vmstack->stack; \
 	stack_item* top_item = vm_stack_top(vmstack); \
 	stack_item* sec_item = vm_stack_second(vmstack); \
 	stack_item_pp2value( top_item ); \
@@ -585,12 +583,20 @@ do { \
 			result_bool = ( sec_item->ival OP top_item->dval ) ; \
 		}else if((top_item->type == DVAL) && (sec_item->type == DVAL)){ \
 			result_bool = ( sec_item->dval OP top_item->dval ) ; \
+		}else{ \
+			result_bool = false; \
+			Rprintf("ERROR: This branch should not be executed.\n"); \
+			vm_error_raise(vmstack); \
+			return 0; \
 		} \
 	}else if(item_is_str(sec_item) && item_is_str(top_item)){ \
 		Rprintf("ERROR: String is not supported for OP calculation.\n"); \
+		vm_error_raise(vmstack); \
 		return 0; \
 	}else{ \
+		result_bool = false; \
 		Rprintf("ERROR: Types are invalied for OP calculation.\n"); \
+		vm_error_raise(vmstack); \
 		return 0; \
 	} \
 	vmstack->sp = vmstack->sp - 1;  \
@@ -627,8 +633,6 @@ vm_calc_le(vm_stack* vmstack)
 int
 vm_calc_neg(vm_stack* vmstack)
 {
-	int sp = vmstack->sp;
-	stack_item* stack = vmstack->stack;
 	stack_item* top_item = vm_stack_top(vmstack);
 	bool result_bool;
 
@@ -639,6 +643,7 @@ vm_calc_neg(vm_stack* vmstack)
 		top_item->boolean = result_bool;
 	}else{
 		Rprintf("ERROR: Type is invalied for VM_NEG command.\n");
+		vm_error_raise(vmstack);
 		return 0;
 	}
 	return 1;
