@@ -189,6 +189,20 @@ typedef union YYSTYPE YYSTYPE;
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
+/* Location type.  */
+#if ! defined YYLTYPE && ! defined YYLTYPE_IS_DECLARED
+typedef struct YYLTYPE YYLTYPE;
+struct YYLTYPE
+{
+  int first_line;
+  int first_column;
+  int last_line;
+  int last_column;
+};
+# define YYLTYPE_IS_DECLARED 1
+# define YYLTYPE_IS_TRIVIAL 1
+#endif
+
 
 
 int yyparse (parser_state *p, void* scanner);
@@ -196,15 +210,18 @@ int yyparse (parser_state *p, void* scanner);
 #endif /* !YY_YY_Y_TAB_H_INCLUDED  */
 
 /* Copy the second part of user declarations.  */
-#line 38 "parse.y" /* yacc.c:358  */
+#line 50 "parse.y" /* yacc.c:358  */
 
 // Prevent undefined reference warnings
 // (ref) https://stackoverflow.com/questions/23717039/generating-a-compiler-from-lex-and-yacc-grammar
 // (ref) https://stackoverflow.com/questions/28643114/how-to-use-flex-with-my-own-parser
-int yylex (YYSTYPE* yylval, parser_state* p, void* scanner);
-void yyerror (parser_state* p, void* scanner, char const *);
+int yylex (YYSTYPE* yylval, YYLTYPE* yylloc, parser_state* p, void* scanner);
+void yyerror (YYLTYPE* loc, parser_state* p, void* scanner, char const *);
+#line 58 "parse.y" /* yacc.c:358  */
 
-#line 207 "y.tab.c" /* yacc.c:358  */
+TreeNode* node_set_loc( TreeNode* nd, YYLTYPE* loc);
+
+#line 224 "y.tab.c" /* yacc.c:358  */
 
 #ifdef short
 # undef short
@@ -385,13 +402,15 @@ void free (void *); /* INFRINGES ON USER NAME SPACE */
 
 #if (! defined yyoverflow \
      && (! defined __cplusplus \
-         || (defined YYSTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))
+         || (defined YYLTYPE_IS_TRIVIAL && YYLTYPE_IS_TRIVIAL \
+             && defined YYSTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))
 
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
   yytype_int16 yyss_alloc;
   YYSTYPE yyvs_alloc;
+  YYLTYPE yyls_alloc;
 };
 
 /* The size of the maximum gap between one aligned stack and the next.  */
@@ -400,8 +419,8 @@ union yyalloc
 /* The size of an array large to enough to hold all stacks, each with
    N elements.  */
 # define YYSTACK_BYTES(N) \
-     ((N) * (sizeof (yytype_int16) + sizeof (YYSTYPE)) \
-      + YYSTACK_GAP_MAXIMUM)
+     ((N) * (sizeof (yytype_int16) + sizeof (YYSTYPE) + sizeof (YYLTYPE)) \
+      + 2 * YYSTACK_GAP_MAXIMUM)
 
 # define YYCOPY_NEEDED 1
 
@@ -504,12 +523,12 @@ static const yytype_uint8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    93,    93,    95,    96,    98,    99,   104,   105,   106,
-     108,   109,   110,   112,   113,   114,   115,   116,   117,   118,
-     119,   120,   121,   122,   123,   124,   125,   126,   127,   128,
-     130,   136,   137,   138,   139,   140,   142,   144,   146,   147,
-     150,   159,   164,   166,   167,   170,   171,   172,   175,   177,
-     184,   185,   187,   188,   190,   191
+       0,   108,   108,   110,   111,   113,   114,   119,   120,   121,
+     123,   124,   125,   127,   128,   129,   130,   131,   132,   133,
+     134,   135,   136,   137,   138,   139,   140,   141,   142,   143,
+     145,   152,   153,   154,   155,   156,   158,   160,   162,   163,
+     166,   175,   180,   182,   183,   186,   187,   188,   191,   193,
+     201,   202,   204,   205,   207,   208
 };
 #endif
 
@@ -703,7 +722,7 @@ do                                                              \
     }                                                           \
   else                                                          \
     {                                                           \
-      yyerror (p, scanner, YY_("syntax error: cannot back up")); \
+      yyerror (&yylloc, p, scanner, YY_("syntax error: cannot back up")); \
       YYERROR;                                                  \
     }                                                           \
 while (0)
@@ -713,6 +732,32 @@ while (0)
 #define YYERRCODE       256
 
 
+/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
+   If N is 0, then set CURRENT to the empty location which ends
+   the previous symbol: RHS[0] (always defined).  */
+
+#ifndef YYLLOC_DEFAULT
+# define YYLLOC_DEFAULT(Current, Rhs, N)                                \
+    do                                                                  \
+      if (N)                                                            \
+        {                                                               \
+          (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;        \
+          (Current).first_column = YYRHSLOC (Rhs, 1).first_column;      \
+          (Current).last_line    = YYRHSLOC (Rhs, N).last_line;         \
+          (Current).last_column  = YYRHSLOC (Rhs, N).last_column;       \
+        }                                                               \
+      else                                                              \
+        {                                                               \
+          (Current).first_line   = (Current).last_line   =              \
+            YYRHSLOC (Rhs, 0).last_line;                                \
+          (Current).first_column = (Current).last_column =              \
+            YYRHSLOC (Rhs, 0).last_column;                              \
+        }                                                               \
+    while (0)
+#endif
+
+#define YYRHSLOC(Rhs, K) ((Rhs)[K])
+
 
 /* Enable debugging if requested.  */
 #if YYDEBUG
@@ -728,9 +773,48 @@ do {                                            \
     YYFPRINTF Args;                             \
 } while (0)
 
-/* This macro is provided for backward compatibility. */
+
+/* YY_LOCATION_PRINT -- Print the location on the stream.
+   This macro was not mandated originally: define only if we know
+   we won't break user code: when these are the locations we know.  */
+
 #ifndef YY_LOCATION_PRINT
-# define YY_LOCATION_PRINT(File, Loc) ((void) 0)
+# if defined YYLTYPE_IS_TRIVIAL && YYLTYPE_IS_TRIVIAL
+
+/* Print *YYLOCP on YYO.  Private, do not rely on its existence. */
+
+YY_ATTRIBUTE_UNUSED
+static unsigned
+yy_location_print_ (FILE *yyo, YYLTYPE const * const yylocp)
+{
+  unsigned res = 0;
+  int end_col = 0 != yylocp->last_column ? yylocp->last_column - 1 : 0;
+  if (0 <= yylocp->first_line)
+    {
+      res += YYFPRINTF (yyo, "%d", yylocp->first_line);
+      if (0 <= yylocp->first_column)
+        res += YYFPRINTF (yyo, ".%d", yylocp->first_column);
+    }
+  if (0 <= yylocp->last_line)
+    {
+      if (yylocp->first_line < yylocp->last_line)
+        {
+          res += YYFPRINTF (yyo, "-%d", yylocp->last_line);
+          if (0 <= end_col)
+            res += YYFPRINTF (yyo, ".%d", end_col);
+        }
+      else if (0 <= end_col && yylocp->first_column < end_col)
+        res += YYFPRINTF (yyo, "-%d", end_col);
+    }
+  return res;
+ }
+
+#  define YY_LOCATION_PRINT(File, Loc)          \
+  yy_location_print_ (File, &(Loc))
+
+# else
+#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
+# endif
 #endif
 
 
@@ -740,7 +824,7 @@ do {                                                                      \
     {                                                                     \
       YYFPRINTF (stderr, "%s ", Title);                                   \
       yy_symbol_print (stderr,                                            \
-                  Type, Value, p, scanner); \
+                  Type, Value, Location, p, scanner); \
       YYFPRINTF (stderr, "\n");                                           \
     }                                                                     \
 } while (0)
@@ -751,10 +835,11 @@ do {                                                                      \
 `----------------------------------------*/
 
 static void
-yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep, parser_state *p, void* scanner)
+yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep, YYLTYPE const * const yylocationp, parser_state *p, void* scanner)
 {
   FILE *yyo = yyoutput;
   YYUSE (yyo);
+  YYUSE (yylocationp);
   YYUSE (p);
   YYUSE (scanner);
   if (!yyvaluep)
@@ -772,12 +857,14 @@ yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvalue
 `--------------------------------*/
 
 static void
-yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep, parser_state *p, void* scanner)
+yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep, YYLTYPE const * const yylocationp, parser_state *p, void* scanner)
 {
   YYFPRINTF (yyoutput, "%s %s (",
              yytype < YYNTOKENS ? "token" : "nterm", yytname[yytype]);
 
-  yy_symbol_value_print (yyoutput, yytype, yyvaluep, p, scanner);
+  YY_LOCATION_PRINT (yyoutput, *yylocationp);
+  YYFPRINTF (yyoutput, ": ");
+  yy_symbol_value_print (yyoutput, yytype, yyvaluep, yylocationp, p, scanner);
   YYFPRINTF (yyoutput, ")");
 }
 
@@ -810,7 +897,7 @@ do {                                                            \
 `------------------------------------------------*/
 
 static void
-yy_reduce_print (yytype_int16 *yyssp, YYSTYPE *yyvsp, int yyrule, parser_state *p, void* scanner)
+yy_reduce_print (yytype_int16 *yyssp, YYSTYPE *yyvsp, YYLTYPE *yylsp, int yyrule, parser_state *p, void* scanner)
 {
   unsigned long int yylno = yyrline[yyrule];
   int yynrhs = yyr2[yyrule];
@@ -824,7 +911,7 @@ yy_reduce_print (yytype_int16 *yyssp, YYSTYPE *yyvsp, int yyrule, parser_state *
       yy_symbol_print (stderr,
                        yystos[yyssp[yyi + 1 - yynrhs]],
                        &(yyvsp[(yyi + 1) - (yynrhs)])
-                                              , p, scanner);
+                       , &(yylsp[(yyi + 1) - (yynrhs)])                       , p, scanner);
       YYFPRINTF (stderr, "\n");
     }
 }
@@ -832,7 +919,7 @@ yy_reduce_print (yytype_int16 *yyssp, YYSTYPE *yyvsp, int yyrule, parser_state *
 # define YY_REDUCE_PRINT(Rule)          \
 do {                                    \
   if (yydebug)                          \
-    yy_reduce_print (yyssp, yyvsp, Rule, p, scanner); \
+    yy_reduce_print (yyssp, yyvsp, yylsp, Rule, p, scanner); \
 } while (0)
 
 /* Nonzero means print parse trace.  It is left uninitialized so that
@@ -1090,9 +1177,10 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 `-----------------------------------------------*/
 
 static void
-yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep, parser_state *p, void* scanner)
+yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep, YYLTYPE *yylocationp, parser_state *p, void* scanner)
 {
   YYUSE (yyvaluep);
+  YYUSE (yylocationp);
   YYUSE (p);
   YYUSE (scanner);
   if (!yymsg)
@@ -1124,6 +1212,14 @@ int yychar;
 YY_INITIAL_VALUE (static YYSTYPE yyval_default;)
 YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
 
+/* Location data for the lookahead symbol.  */
+static YYLTYPE yyloc_default
+# if defined YYLTYPE_IS_TRIVIAL && YYLTYPE_IS_TRIVIAL
+  = { 1, 1, 1, 1 }
+# endif
+;
+YYLTYPE yylloc = yyloc_default;
+
     /* Number of syntax errors so far.  */
     int yynerrs;
 
@@ -1134,6 +1230,7 @@ YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
     /* The stacks and their tools:
        'yyss': related to states.
        'yyvs': related to semantic values.
+       'yyls': related to locations.
 
        Refer to the stacks through separate pointers, to allow yyoverflow
        to reallocate them elsewhere.  */
@@ -1148,6 +1245,14 @@ YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
     YYSTYPE *yyvs;
     YYSTYPE *yyvsp;
 
+    /* The location stack.  */
+    YYLTYPE yylsa[YYINITDEPTH];
+    YYLTYPE *yyls;
+    YYLTYPE *yylsp;
+
+    /* The locations where the error started and ended.  */
+    YYLTYPE yyerror_range[3];
+
     YYSIZE_T yystacksize;
 
   int yyn;
@@ -1157,6 +1262,7 @@ YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
   /* The variables used to return semantic value and location from the
      action routines.  */
   YYSTYPE yyval;
+  YYLTYPE yyloc;
 
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
@@ -1165,7 +1271,7 @@ YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
   YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
 #endif
 
-#define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))
+#define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N), yylsp -= (N))
 
   /* The number of symbols on the RHS of the reduced rule.
      Keep to zero when no symbol should be popped.  */
@@ -1173,6 +1279,7 @@ YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
 
   yyssp = yyss = yyssa;
   yyvsp = yyvs = yyvsa;
+  yylsp = yyls = yylsa;
   yystacksize = YYINITDEPTH;
 
   YYDPRINTF ((stderr, "Starting parse\n"));
@@ -1181,6 +1288,7 @@ YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
   yyerrstatus = 0;
   yynerrs = 0;
   yychar = YYEMPTY; /* Cause a token to be read.  */
+  yylsp[0] = yylloc;
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -1206,6 +1314,7 @@ YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
            memory.  */
         YYSTYPE *yyvs1 = yyvs;
         yytype_int16 *yyss1 = yyss;
+        YYLTYPE *yyls1 = yyls;
 
         /* Each stack pointer address is followed by the size of the
            data in use in that stack, in bytes.  This used to be a
@@ -1214,8 +1323,10 @@ YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
         yyoverflow (YY_("memory exhausted"),
                     &yyss1, yysize * sizeof (*yyssp),
                     &yyvs1, yysize * sizeof (*yyvsp),
+                    &yyls1, yysize * sizeof (*yylsp),
                     &yystacksize);
 
+        yyls = yyls1;
         yyss = yyss1;
         yyvs = yyvs1;
       }
@@ -1238,6 +1349,7 @@ YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
           goto yyexhaustedlab;
         YYSTACK_RELOCATE (yyss_alloc, yyss);
         YYSTACK_RELOCATE (yyvs_alloc, yyvs);
+        YYSTACK_RELOCATE (yyls_alloc, yyls);
 #  undef YYSTACK_RELOCATE
         if (yyss1 != yyssa)
           YYSTACK_FREE (yyss1);
@@ -1247,6 +1359,7 @@ YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
 
       yyssp = yyss + yysize - 1;
       yyvsp = yyvs + yysize - 1;
+      yylsp = yyls + yysize - 1;
 
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
                   (unsigned long int) yystacksize));
@@ -1281,7 +1394,7 @@ yybackup:
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
-      yychar = yylex (&yylval, p, scanner);
+      yychar = yylex (&yylval, &yylloc, p, scanner);
     }
 
   if (yychar <= YYEOF)
@@ -1324,7 +1437,7 @@ yybackup:
   YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   *++yyvsp = yylval;
   YY_IGNORE_MAYBE_UNINITIALIZED_END
-
+  *++yylsp = yylloc;
   goto yynewstate;
 
 
@@ -1355,316 +1468,319 @@ yyreduce:
      GCC warning that YYVAL may be used uninitialized.  */
   yyval = yyvsp[1-yylen];
 
-
+  /* Default location.  */
+  YYLLOC_DEFAULT (yyloc, (yylsp - yylen), yylen);
   YY_REDUCE_PRINT (yyn);
   switch (yyn)
     {
         case 2:
-#line 93 "parse.y" /* yacc.c:1646  */
+#line 108 "parse.y" /* yacc.c:1646  */
     { p->tree = new_node_prgm( (yyvsp[0].nd) );}
-#line 1365 "y.tab.c" /* yacc.c:1646  */
+#line 1479 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 3:
-#line 95 "parse.y" /* yacc.c:1646  */
+#line 110 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = (yyvsp[-1].nd);  }
-#line 1371 "y.tab.c" /* yacc.c:1646  */
+#line 1485 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 4:
-#line 96 "parse.y" /* yacc.c:1646  */
+#line 111 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = NULL; }
-#line 1377 "y.tab.c" /* yacc.c:1646  */
+#line 1491 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 5:
-#line 98 "parse.y" /* yacc.c:1646  */
+#line 113 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = new_node_stmt((yyvsp[0].nd)); }
-#line 1383 "y.tab.c" /* yacc.c:1646  */
+#line 1497 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 6:
-#line 100 "parse.y" /* yacc.c:1646  */
+#line 115 "parse.y" /* yacc.c:1646  */
     {
 					(yyval.nd) = pushback_node_stmt((yyvsp[-2].nd), new_node_stmt((yyvsp[0].nd)));
 					}
-#line 1391 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 7:
-#line 104 "parse.y" /* yacc.c:1646  */
-    { /*printf("ASSIGN STMT!!!"); */ (yyval.nd) = (yyvsp[0].nd); }
-#line 1397 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 8:
-#line 105 "parse.y" /* yacc.c:1646  */
-    { /*printf("IF STMT!!!"); */ (yyval.nd) = (yyvsp[0].nd); }
-#line 1403 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 9:
-#line 106 "parse.y" /* yacc.c:1646  */
-    { /*printf("JUST STMT!!!"); */ (yyval.nd) = (yyvsp[0].nd); }
-#line 1409 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 10:
-#line 108 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("AND", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1415 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 11:
-#line 109 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("OR", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1421 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 12:
-#line 110 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = (yyvsp[0].nd); }
-#line 1427 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 13:
-#line 112 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = (yyvsp[0].nd); }
-#line 1433 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 14:
-#line 113 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("PLUS", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1439 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 15:
-#line 114 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("SUB", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1445 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 16:
-#line 115 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("MULT", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1451 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 17:
-#line 116 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("DIV", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1457 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 18:
-#line 117 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("MOD", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1463 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 19:
-#line 118 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_uniop("FACTOR", (yyvsp[-1].nd)); }
-#line 1469 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 20:
-#line 119 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("POWER", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1475 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 21:
-#line 120 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("EQ", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1481 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 22:
-#line 121 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("NEQ", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1487 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 23:
-#line 122 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("GT", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1493 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 24:
-#line 123 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("LT", (yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1499 "y.tab.c" /* yacc.c:1646  */
-    break;
-
-  case 25:
-#line 124 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("GE", (yyvsp[-2].nd), (yyvsp[0].nd)); }
 #line 1505 "y.tab.c" /* yacc.c:1646  */
     break;
 
-  case 26:
-#line 125 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("LE", (yyvsp[-2].nd), (yyvsp[0].nd)); }
+  case 7:
+#line 119 "parse.y" /* yacc.c:1646  */
+    { /*printf("ASSIGN STMT!!!"); */ (yyval.nd) = (yyvsp[0].nd); }
 #line 1511 "y.tab.c" /* yacc.c:1646  */
     break;
 
-  case 27:
-#line 126 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_uniop("UMINUS", (yyvsp[0].nd)); }
+  case 8:
+#line 120 "parse.y" /* yacc.c:1646  */
+    { /*printf("IF STMT!!!"); */ (yyval.nd) = (yyvsp[0].nd); }
 #line 1517 "y.tab.c" /* yacc.c:1646  */
     break;
 
-  case 28:
-#line 127 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_op("REXP_MATCH", (yyvsp[-2].nd), (yyvsp[0].nd)); }
+  case 9:
+#line 121 "parse.y" /* yacc.c:1646  */
+    { /*printf("JUST STMT!!!"); */ (yyval.nd) = (yyvsp[0].nd); }
 #line 1523 "y.tab.c" /* yacc.c:1646  */
     break;
 
-  case 29:
-#line 128 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = (yyvsp[0].nd); }
+  case 10:
+#line 123 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("AND", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
 #line 1529 "y.tab.c" /* yacc.c:1646  */
     break;
 
-  case 30:
+  case 11:
+#line 124 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("OR", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1535 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 12:
+#line 125 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = (yyvsp[0].nd); }
+#line 1541 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 13:
+#line 127 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = (yyvsp[0].nd); }
+#line 1547 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 14:
+#line 128 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("PLUS", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1553 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 15:
+#line 129 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("SUB", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1559 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 16:
+#line 130 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("MULT", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1565 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 17:
 #line 131 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("DIV", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1571 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 18:
+#line 132 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("MOD", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1577 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 19:
+#line 133 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_uniop("FACTOR", (yyvsp[-1].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1583 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 20:
+#line 134 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("POWER", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1589 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 21:
+#line 135 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("EQ", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1595 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 22:
+#line 136 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("NEQ", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1601 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 23:
+#line 137 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("GT", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1607 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 24:
+#line 138 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("LT", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1613 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 25:
+#line 139 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("GE", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1619 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 26:
+#line 140 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("LE", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1625 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 27:
+#line 141 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_uniop("UMINUS", (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1631 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 28:
+#line 142 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_op("REXP_MATCH", (yyvsp[-2].nd), (yyvsp[0].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1637 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 29:
+#line 143 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = (yyvsp[0].nd); }
+#line 1643 "y.tab.c" /* yacc.c:1646  */
+    break;
+
+  case 30:
+#line 146 "parse.y" /* yacc.c:1646  */
     {
 			(yyval.nd) = new_node_ident( (yyvsp[0].id) ); 
 			var_hash_add_name( &(p->vars) , (yyvsp[0].id) );
 			var_hash_add_name( &(p->rhsvars) , (yyvsp[0].id) );
+			(yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); 
 			}
-#line 1539 "y.tab.c" /* yacc.c:1646  */
+#line 1654 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 31:
-#line 136 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = (yyvsp[0].nd); }
-#line 1545 "y.tab.c" /* yacc.c:1646  */
+#line 152 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = (yyvsp[0].nd); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1660 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 32:
-#line 137 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_str( (yyvsp[0].str) , p->ptrtable ); }
-#line 1551 "y.tab.c" /* yacc.c:1646  */
+#line 153 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_str( (yyvsp[0].str) , p->ptrtable ); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1666 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 33:
-#line 138 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_rexp( (yyvsp[0].str) , p->ptrtable , p->rexp_encoding ); }
-#line 1557 "y.tab.c" /* yacc.c:1646  */
+#line 154 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_rexp( (yyvsp[0].str) , p->ptrtable , p->rexp_encoding ); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1672 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 34:
-#line 139 "parse.y" /* yacc.c:1646  */
+#line 155 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = (yyvsp[-1].nd); }
-#line 1563 "y.tab.c" /* yacc.c:1646  */
+#line 1678 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 35:
-#line 140 "parse.y" /* yacc.c:1646  */
+#line 156 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = (yyvsp[0].nd); }
-#line 1569 "y.tab.c" /* yacc.c:1646  */
+#line 1684 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 36:
-#line 142 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_fcall((yyvsp[-3].nd), (yyvsp[-1].nd)); }
-#line 1575 "y.tab.c" /* yacc.c:1646  */
+#line 158 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_fcall((yyvsp[-3].nd), (yyvsp[-1].nd)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1690 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 37:
-#line 144 "parse.y" /* yacc.c:1646  */
-    { (yyval.nd) = new_node_ident((yyvsp[0].id)); }
-#line 1581 "y.tab.c" /* yacc.c:1646  */
+#line 160 "parse.y" /* yacc.c:1646  */
+    { (yyval.nd) = new_node_ident((yyvsp[0].id)); (yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); }
+#line 1696 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 38:
-#line 146 "parse.y" /* yacc.c:1646  */
+#line 162 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = new_node_null(); }
-#line 1587 "y.tab.c" /* yacc.c:1646  */
+#line 1702 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 39:
-#line 147 "parse.y" /* yacc.c:1646  */
+#line 163 "parse.y" /* yacc.c:1646  */
     { 
 								(yyval.nd) = new_node_farg((yyvsp[0].nd)); 
 								}
-#line 1595 "y.tab.c" /* yacc.c:1646  */
+#line 1710 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 40:
-#line 151 "parse.y" /* yacc.c:1646  */
+#line 167 "parse.y" /* yacc.c:1646  */
     {
 					(yyval.nd) = pushback_node_farg((yyvsp[-2].nd), new_node_farg((yyvsp[0].nd)));
 					}
-#line 1603 "y.tab.c" /* yacc.c:1646  */
+#line 1718 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 41:
-#line 160 "parse.y" /* yacc.c:1646  */
+#line 176 "parse.y" /* yacc.c:1646  */
     {
 					(yyval.nd) = new_node_if( (yyvsp[-2].nd), (yyvsp[-1].nd), (yyvsp[0].nd) );
 					}
-#line 1611 "y.tab.c" /* yacc.c:1646  */
+#line 1726 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 42:
-#line 164 "parse.y" /* yacc.c:1646  */
+#line 180 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = (yyvsp[-2].nd) ; }
-#line 1617 "y.tab.c" /* yacc.c:1646  */
+#line 1732 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 43:
-#line 166 "parse.y" /* yacc.c:1646  */
+#line 182 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = (yyvsp[-1].nd); }
-#line 1623 "y.tab.c" /* yacc.c:1646  */
+#line 1738 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 44:
-#line 167 "parse.y" /* yacc.c:1646  */
+#line 183 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = (yyvsp[-1].nd); }
-#line 1629 "y.tab.c" /* yacc.c:1646  */
+#line 1744 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 45:
-#line 170 "parse.y" /* yacc.c:1646  */
+#line 186 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = NULL; }
-#line 1635 "y.tab.c" /* yacc.c:1646  */
+#line 1750 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 46:
-#line 171 "parse.y" /* yacc.c:1646  */
+#line 187 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = (yyvsp[0].nd); }
-#line 1641 "y.tab.c" /* yacc.c:1646  */
+#line 1756 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 47:
-#line 172 "parse.y" /* yacc.c:1646  */
+#line 188 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = (yyvsp[-1].nd); }
-#line 1647 "y.tab.c" /* yacc.c:1646  */
+#line 1762 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 48:
-#line 175 "parse.y" /* yacc.c:1646  */
+#line 191 "parse.y" /* yacc.c:1646  */
     { (yyval.nd) = new_node_let((yyvsp[-2].nd), (yyvsp[0].nd)); }
-#line 1653 "y.tab.c" /* yacc.c:1646  */
+#line 1768 "y.tab.c" /* yacc.c:1646  */
     break;
 
   case 49:
-#line 178 "parse.y" /* yacc.c:1646  */
+#line 194 "parse.y" /* yacc.c:1646  */
     {
 					(yyval.nd) = new_node_ident( (yyvsp[0].id) );
 					var_hash_add_name( &(p->vars) , (yyvsp[0].id) );
 					var_hash_add_name( &(p->lhsvars) , (yyvsp[0].id) );
+					(yyval.nd) = node_set_loc( (yyval.nd), &(yyloc) ); 
 					}
-#line 1663 "y.tab.c" /* yacc.c:1646  */
+#line 1779 "y.tab.c" /* yacc.c:1646  */
     break;
 
 
-#line 1667 "y.tab.c" /* yacc.c:1646  */
+#line 1783 "y.tab.c" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -1685,6 +1801,7 @@ yyreduce:
   YY_STACK_PRINT (yyss, yyssp);
 
   *++yyvsp = yyval;
+  *++yylsp = yyloc;
 
   /* Now 'shift' the result of the reduction.  Determine what state
      that goes to, based on the state we popped back to and the rule
@@ -1714,7 +1831,7 @@ yyerrlab:
     {
       ++yynerrs;
 #if ! YYERROR_VERBOSE
-      yyerror (p, scanner, YY_("syntax error"));
+      yyerror (&yylloc, p, scanner, YY_("syntax error"));
 #else
 # define YYSYNTAX_ERROR yysyntax_error (&yymsg_alloc, &yymsg, \
                                         yyssp, yytoken)
@@ -1741,7 +1858,7 @@ yyerrlab:
                 yymsgp = yymsg;
               }
           }
-        yyerror (p, scanner, yymsgp);
+        yyerror (&yylloc, p, scanner, yymsgp);
         if (yysyntax_error_status == 2)
           goto yyexhaustedlab;
       }
@@ -1749,7 +1866,7 @@ yyerrlab:
 #endif
     }
 
-
+  yyerror_range[1] = yylloc;
 
   if (yyerrstatus == 3)
     {
@@ -1765,7 +1882,7 @@ yyerrlab:
       else
         {
           yydestruct ("Error: discarding",
-                      yytoken, &yylval, p, scanner);
+                      yytoken, &yylval, &yylloc, p, scanner);
           yychar = YYEMPTY;
         }
     }
@@ -1786,6 +1903,7 @@ yyerrorlab:
   if (/*CONSTCOND*/ 0)
      goto yyerrorlab;
 
+  yyerror_range[1] = yylsp[1-yylen];
   /* Do not reclaim the symbols of the rule whose action triggered
      this YYERROR.  */
   YYPOPSTACK (yylen);
@@ -1819,9 +1937,9 @@ yyerrlab1:
       if (yyssp == yyss)
         YYABORT;
 
-
+      yyerror_range[1] = *yylsp;
       yydestruct ("Error: popping",
-                  yystos[yystate], yyvsp, p, scanner);
+                  yystos[yystate], yyvsp, yylsp, p, scanner);
       YYPOPSTACK (1);
       yystate = *yyssp;
       YY_STACK_PRINT (yyss, yyssp);
@@ -1831,6 +1949,11 @@ yyerrlab1:
   *++yyvsp = yylval;
   YY_IGNORE_MAYBE_UNINITIALIZED_END
 
+  yyerror_range[2] = yylloc;
+  /* Using YYLLOC is tempting, but would change the location of
+     the lookahead.  YYLOC is available though.  */
+  YYLLOC_DEFAULT (yyloc, yyerror_range, 2);
+  *++yylsp = yyloc;
 
   /* Shift the error token.  */
   YY_SYMBOL_PRINT ("Shifting", yystos[yyn], yyvsp, yylsp);
@@ -1858,7 +1981,7 @@ yyabortlab:
 | yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
 yyexhaustedlab:
-  yyerror (p, scanner, YY_("memory exhausted"));
+  yyerror (&yylloc, p, scanner, YY_("memory exhausted"));
   yyresult = 2;
   /* Fall through.  */
 #endif
@@ -1870,7 +1993,7 @@ yyreturn:
          user semantic actions for why this is necessary.  */
       yytoken = YYTRANSLATE (yychar);
       yydestruct ("Cleanup: discarding lookahead",
-                  yytoken, &yylval, p, scanner);
+                  yytoken, &yylval, &yylloc, p, scanner);
     }
   /* Do not reclaim the symbols of the rule whose action triggered
      this YYABORT or YYACCEPT.  */
@@ -1879,7 +2002,7 @@ yyreturn:
   while (yyssp != yyss)
     {
       yydestruct ("Cleanup: popping",
-                  yystos[*yyssp], yyvsp, p, scanner);
+                  yystos[*yyssp], yyvsp, yylsp, p, scanner);
       YYPOPSTACK (1);
     }
 #ifndef yyoverflow
@@ -1892,14 +2015,22 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 194 "parse.y" /* yacc.c:1906  */
+#line 211 "parse.y" /* yacc.c:1906  */
 
 
-void yyerror(parser_state* p, void* scanner, char const* s)
+void yyerror(YYLTYPE* loc , parser_state* p, void* scanner, char const* message)
 {
   p->yynerrs++;
-  fprintf(stderr, "%d: %s\n", p->lineno, s);
-  /* yylineno does not seem to work in reentrant parser, b/c it's global.*/
-  /* Instead, the line number is set in parser_state by tokenizer or scanner. */
+  fprintf(stderr, "%s (near line: %d column: %d )\n", message , loc->first_line, loc->first_column );
+}
+
+TreeNode*
+node_set_loc( TreeNode* nd, YYLTYPE* loc)
+{
+  nd->loc.first_line   = loc->first_line;
+  nd->loc.first_column = loc->first_column;
+  nd->loc.last_line    = loc->last_line;
+  nd->loc.last_column  = loc->last_column;
+  return nd;
 }
 
